@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package YAML::PP::Loader;
 
-our $VERSION = '0.001_001'; # TRIAL VERSION
+our $VERSION = '0.001_002'; # TRIAL VERSION
 
 use YAML::PP::Parser;
 
@@ -25,6 +25,12 @@ sub new {
     elsif ($bool eq 'perl') {
         $truefalse = \&bool_perl;
     }
+    else {
+        die "Invalid value for 'boolean': '$bool'. Allowed: ('perl', 'boolean', 'JSON::PP')";
+    }
+    if (keys %args) {
+        die "Unexpected arguments: " . join ', ', sort keys %args;
+    }
     my $self = bless {
         boolean => $bool,
         truefalse => $truefalse,
@@ -60,7 +66,7 @@ sub Load {
 sub event {
     my ($self, $parser, $event, $docs) = @_;
     my ($name, $info) = @$event;
-    DEBUG and warn "event($name)\n";
+    DEBUG and warn YAML::PP::Parser->event_to_test_suite($event) ."\n";
 
     my $refs = $self->refs;
     if ($name eq 'BEGIN') {
@@ -82,6 +88,11 @@ sub event {
                 push @$$ref, $data;
                 push @$refs, \$data;
             }
+            elsif (ref $$ref eq 'HASH') {
+                # we got a complex key
+                push @$refs, \\undef;
+                push @$refs, \$data;
+            }
             else {
                 die "Unexpected";
             }
@@ -91,10 +102,32 @@ sub event {
         }
     }
     elsif ($name eq 'END') {
-        if ($info->{type} eq 'DOC') {
+        my $type = $info->{type};
+        if ($type eq 'DOC') {
             push @$docs, $self->data;
+            pop @$refs if @$refs;
+        }
+        elsif ($type eq 'MAP' or $type eq 'SEQ') {
+            my $complex = pop @$refs;
+            if (@$refs > 1) {
+                my $ref1 = $refs->[-1];
+                my $ref2 = $refs->[-2];
+                if (ref $$ref1 eq 'SCALAR') {
+                    pop @$refs;
+                    my $string = $self->stringify_complex($$complex);
+                    if (ref $$ref2 eq 'HASH') {
+                        $$ref2->{ $string } = undef;
+                        push @$refs, \$$ref2->{ $string };
+                    }
+                    else {
+                        die "Unexpected";
+                    }
+                }
+            }
+        }
+        else {
+            pop @$refs if @$refs;
         }
-        pop @$refs if @$refs;
     }
     elsif ($name eq 'VALUE' or $name eq 'ALIAS') {
         my $value;
@@ -142,6 +175,19 @@ sub render_value {
     return $value;
 }
 
+sub stringify_complex {
+    my ($self, $data) = @_;
+    require Data::Dumper;
+    local $Data::Dumper::Quotekeys = 0;
+    local $Data::Dumper::Terse = 1;
+    local $Data::Dumper::Indent = 0;
+    local $Data::Dumper::Useqq = 0;
+    local $Data::Dumper::Sortkeys = 1;
+    my $string = Data::Dumper->Dump([$data], ['data']);
+    $string =~ s/^\$data = //;
+    return $string;
+}
+
 sub render_plain_scalar {
     my ($self, $content) = @_;
     return unless defined $content;
